@@ -61,7 +61,7 @@ import org.eclipse.jgit.lib.ObjectLoader;
 /**
  * Unix TAR format (ustar + some PAX extensions).
  */
-public class TarFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
+public final class TarFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
 	private static final List<String> SUFFIXES = Collections
 			.unmodifiableList(Arrays.asList(".tar")); //$NON-NLS-1$
 
@@ -87,11 +87,11 @@ public class TarFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
 
 		// TarArchiveEntry detects directories by checking
 		// for '/' at the end of the filename.
-		if (path.endsWith("/") && mode != FileMode.TREE)
+		if (path.endsWith("/") && mode != FileMode.TREE) //$NON-NLS-1$
 			throw new IllegalArgumentException(MessageFormat.format(
 					ArchiveText.get().pathDoesNotMatchMode, path, mode));
-		if (!path.endsWith("/") && mode == FileMode.TREE)
-			path = path + "/";
+		if (!path.endsWith("/") && mode == FileMode.TREE) //$NON-NLS-1$
+			path = path + "/"; //$NON-NLS-1$
 
 		final TarArchiveEntry entry = new TarArchiveEntry(path);
 		if (mode == FileMode.TREE) {
@@ -118,4 +118,14 @@ public class TarFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
 	public Iterable<String> suffixes() {
 		return SUFFIXES;
 	}
+
+	@Override
+	public boolean equals(Object other) {
+		return (other instanceof TarFormat);
+	}
+
+	@Override
+	public int hashCode() {
+		return getClass().hashCode();
+	}
 }
