@@ -57,7 +57,7 @@ import org.eclipse.jgit.lib.ObjectLoader;
 /**
  * bzip2-compressed tarball (tar.bz2) format.
  */
-public class Tbz2Format implements ArchiveCommand.Format<ArchiveOutputStream> {
+public final class Tbz2Format implements ArchiveCommand.Format<ArchiveOutputStream> {
 	private static final List<String> SUFFIXES = Collections
 			.unmodifiableList(Arrays.asList(".tar.bz2", ".tbz", ".tbz2")); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
 
@@ -78,4 +78,14 @@ public class Tbz2Format implements ArchiveCommand.Format<ArchiveOutputStream> {
 	public Iterable<String> suffixes() {
 		return SUFFIXES;
 	}
+
+	@Override
+	public boolean equals(Object other) {
+		return (other instanceof Tbz2Format);
+	}
+
+	@Override
+	public int hashCode() {
+		return getClass().hashCode();
+	}
 }
