@@ -57,7 +57,7 @@ import org.eclipse.jgit.lib.ObjectLoader;
 /**
  * gzip-compressed tarball (tar.gz) format.
  */
-public class TgzFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
+public final class TgzFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
 	private static final List<String> SUFFIXES = Collections
 			.unmodifiableList(Arrays.asList(".tar.gz", ".tgz")); //$NON-NLS-1$ //$NON-NLS-2$
 
@@ -78,4 +78,14 @@ public class TgzFormat implements ArchiveCommand.Format<ArchiveOutputStream> {
 	public Iterable<String> suffixes() {
 		return SUFFIXES;
 	}
+
+	@Override
+	public boolean equals(Object other) {
+		return (other instanceof TgzFormat);
+	}
+
+	@Override
+	public int hashCode() {
+		return getClass().hashCode();
+	}
 }
