@@ -304,7 +304,7 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 
 		final byte[] result = CLIGitCommand.rawExecute( //
 				"git archive --format=zip master", db);
-		String[] expect = { "a", "b.c", "b0c", "b/a", "b/b", "c" };
+		String[] expect = { "a", "b.c", "b0c", "b/", "b/a", "b/b", "c" };
 		String[] actual = listZipEntries(result);
 
 		Arrays.sort(expect);
@@ -330,7 +330,140 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 
 		final byte[] result = CLIGitCommand.rawExecute( //
 				"git archive --format=tar master", db);
-		String[] expect = { "a", "b.c", "b0c", "b/a", "b/b", "c" };
+		String[] expect = { "a", "b.c", "b0c", "b/", "b/a", "b/b", "c" };
+		String[] actual = listTarEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	private void commitBazAndFooSlashBar() throws Exception {
+		writeTrashFile("baz", "a file");
+		writeTrashFile("foo/bar", "another file");
+		git.add().addFilepattern("baz").call();
+		git.add().addFilepattern("foo").call();
+		git.commit().setMessage("sample commit").call();
+	}
+
+	@Test
+	public void testArchivePrefixOption() throws Exception {
+		commitBazAndFooSlashBar();
+		byte[] result = CLIGitCommand.rawExecute(
+				"git archive --prefix=x/ --format=zip master", db);
+		String[] expect = { "x/baz", "x/foo/", "x/foo/bar" };
+		String[] actual = listZipEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	@Test
+	public void testTarPrefixOption() throws Exception {
+		commitBazAndFooSlashBar();
+		byte[] result = CLIGitCommand.rawExecute(
+				"git archive --prefix=x/ --format=tar master", db);
+		String[] expect = { "x/baz", "x/foo/", "x/foo/bar" };
+		String[] actual = listTarEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	private void commitFoo() throws Exception {
+		writeTrashFile("foo", "a file");
+		git.add().addFilepattern("foo").call();
+		git.commit().setMessage("boring commit").call();
+	}
+
+	@Test
+	public void testPrefixDoesNotNormalizeDoubleSlash() throws Exception {
+		commitFoo();
+		byte[] result = CLIGitCommand.rawExecute(
+				"git archive --prefix=x// --format=zip master", db);
+		String[] expect = { "x//foo" };
+		assertArrayEquals(expect, listZipEntries(result));
+	}
+
+	@Test
+	public void testPrefixDoesNotNormalizeDoubleSlashInTar() throws Exception {
+		commitFoo();
+		final byte[] result = CLIGitCommand.rawExecute( //
+				"git archive --prefix=x// --format=tar master", db);
+		String[] expect = { "x//foo" };
+		assertArrayEquals(expect, listTarEntries(result));
+	}
+
+	/**
+	 * The prefix passed to "git archive" need not end with '/'.
+	 * In practice it is not very common to have a nonempty prefix
+	 * that does not name a directory (and hence end with /), but
+	 * since git has historically supported other prefixes, we do,
+	 * too.
+	 *
+	 * @throws Exception
+	 */
+	@Test
+	public void testPrefixWithoutTrailingSlash() throws Exception {
+		commitBazAndFooSlashBar();
+		byte[] result = CLIGitCommand.rawExecute(
+				"git archive --prefix=my- --format=zip master", db);
+		String[] expect = { "my-baz", "my-foo/", "my-foo/bar" };
+		String[] actual = listZipEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	@Test
+	public void testTarPrefixWithoutTrailingSlash() throws Exception {
+		commitBazAndFooSlashBar();
+		final byte[] result = CLIGitCommand.rawExecute( //
+				"git archive --prefix=my- --format=tar master", db);
+		String[] expect = { "my-baz", "my-foo/", "my-foo/bar" };
+		String[] actual = listTarEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	@Test
+	public void testArchiveIncludesSubmoduleDirectory() throws Exception {
+		writeTrashFile("a", "a file with content!");
+		writeTrashFile("c", "after submodule");
+		git.add().addFilepattern("a").call();
+		git.add().addFilepattern("c").call();
+		git.commit().setMessage("initial commit").call();
+		git.submoduleAdd().setURI("./.").setPath("b").call().close();
+		git.commit().setMessage("add submodule").call();
+
+		final byte[] result = CLIGitCommand.rawExecute( //
+				"git archive --format=zip master", db);
+		String[] expect = { ".gitmodules", "a", "b/", "c" };
+		String[] actual = listZipEntries(result);
+
+		Arrays.sort(expect);
+		Arrays.sort(actual);
+		assertArrayEquals(expect, actual);
+	}
+
+	@Test
+	public void testTarIncludesSubmoduleDirectory() throws Exception {
+		writeTrashFile("a", "a file with content!");
+		writeTrashFile("c", "after submodule");
+		git.add().addFilepattern("a").call();
+		git.add().addFilepattern("c").call();
+		git.commit().setMessage("initial commit").call();
+		git.submoduleAdd().setURI("./.").setPath("b").call().close();
+		git.commit().setMessage("add submodule").call();
+
+		final byte[] result = CLIGitCommand.rawExecute( //
+				"git archive --format=tar master", db);
+		String[] expect = { ".gitmodules", "a", "b/", "c" };
 		String[] actual = listTarEntries(result);
 
 		Arrays.sort(expect);
@@ -343,9 +476,11 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 		writeTrashFile("plain", "a file with content");
 		writeTrashFile("executable", "an executable file");
 		writeTrashFile("symlink", "plain");
+		writeTrashFile("dir/content", "clutter in a subdir");
 		git.add().addFilepattern("plain").call();
 		git.add().addFilepattern("executable").call();
 		git.add().addFilepattern("symlink").call();
+		git.add().addFilepattern("dir").call();
 
 		DirCache cache = db.lockDirCache();
 		cache.getEntry("executable").setFileMode(FileMode.EXECUTABLE_FILE);
@@ -362,6 +497,7 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 		assertContainsEntryWithMode("zip-with-modes.zip", "-rw-", "plain");
 		assertContainsEntryWithMode("zip-with-modes.zip", "-rwx", "executable");
 		assertContainsEntryWithMode("zip-with-modes.zip", "l", "symlink");
+		assertContainsEntryWithMode("zip-with-modes.zip", "-rw-", "dir/");
 	}
 
 	@Test
@@ -369,9 +505,11 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 		writeTrashFile("plain", "a file with content");
 		writeTrashFile("executable", "an executable file");
 		writeTrashFile("symlink", "plain");
+		writeTrashFile("dir/content", "clutter in a subdir");
 		git.add().addFilepattern("plain").call();
 		git.add().addFilepattern("executable").call();
 		git.add().addFilepattern("symlink").call();
+		git.add().addFilepattern("dir").call();
 
 		DirCache cache = db.lockDirCache();
 		cache.getEntry("executable").setFileMode(FileMode.EXECUTABLE_FILE);
@@ -388,35 +526,46 @@ public class ArchiveTest extends CLIRepositoryTestCase {
 		assertTarContainsEntry("with-modes.tar", "-rw-r--r--", "plain");
 		assertTarContainsEntry("with-modes.tar", "-rwxr-xr-x", "executable");
 		assertTarContainsEntry("with-modes.tar", "l", "symlink -> plain");
+		assertTarContainsEntry("with-modes.tar", "drwxr-xr-x", "dir/");
 	}
 
 	@Test
 	public void testArchiveWithLongFilename() throws Exception {
-		String filename = "1234567890";
-		for (int i = 0; i < 20; i++)
-			filename = filename + "/1234567890";
+		String filename = "";
+		final List<String> l = new ArrayList<String>();
+		for (int i = 0; i < 20; i++) {
+			filename = filename + "1234567890/";
+			l.add(filename);
+		}
+		filename = filename + "1234567890";
+		l.add(filename);
 		writeTrashFile(filename, "file with long path");
 		git.add().addFilepattern("1234567890").call();
 		git.commit().setMessage("file with long name").call();
 
 		final byte[] result = CLIGitCommand.rawExecute( //
 				"git archive --format=zip HEAD", db);
-		assertArrayEquals(new String[] { filename },
+		assertArrayEquals(l.toArray(new String[l.size()]),
 				listZipEntries(result));
 	}
 
 	@Test
 	public void testTarWithLongFilename() throws Exception {
-		String filename = "1234567890";
-		for (int i = 0; i < 20; i++)
-			filename = filename + "/1234567890";
+		String filename = "";
+		final List<String> l = new ArrayList<String>();
+		for (int i = 0; i < 20; i++) {
+			filename = filename + "1234567890/";
+			l.add(filename);
+		}
+		filename = filename + "1234567890";
+		l.add(filename);
 		writeTrashFile(filename, "file with long path");
 		git.add().addFilepattern("1234567890").call();
 		git.commit().setMessage("file with long name").call();
 
 		final byte[] result = CLIGitCommand.rawExecute( //
 				"git archive --format=tar HEAD", db);
-		assertArrayEquals(new String[] { filename },
+		assertArrayEquals(l.toArray(new String[l.size()]),
 				listTarEntries(result));
 	}
 
