@@ -110,14 +110,14 @@ class RebuildCommitGraph extends TextBuiltin {
 	@Argument(index = 1, required = true, metaVar = "metaVar_refs", usage = "usage_logAllPretty")
 	File graph;
 
-	private final ProgressMonitor pm = new TextProgressMonitor();
+	private final ProgressMonitor pm = new TextProgressMonitor(errw);
 
 	private Map<ObjectId, ObjectId> rewrites = new HashMap<ObjectId, ObjectId>();
 
 	@Override
 	protected void run() throws Exception {
 		if (!really && !db.getRefDatabase().getRefs(ALL).isEmpty()) {
-			System.err.println(
+			errw.println(
 				MessageFormat.format(CLIText.get().fatalThisProgramWillDestroyTheRepository
 					, db.getDirectory().getAbsolutePath(), REALLY));
 			throw die(CLIText.get().needApprovalToDestroyCurrentRepository);
@@ -294,7 +294,7 @@ class RebuildCommitGraph extends TextBuiltin {
 					rw.parseAny(id);
 				} catch (MissingObjectException mue) {
 					if (!Constants.TYPE_COMMIT.equals(type)) {
-						System.err.println(MessageFormat.format(CLIText.get().skippingObject, type, name));
+						errw.println(MessageFormat.format(CLIText.get().skippingObject, type, name));
 						continue;
 					}
 					throw new MissingObjectException(id, type);
