@@ -1225,6 +1225,27 @@ public class RefDirectoryTest extends LocalDiskRepositoryTestCase {
 		assertEquals(A.getId(), refs.get("refs/heads/masters").getObjectId());
 	}
 
+	@Test
+	public void testBatchRefUpdateNonFastForwardDoesNotDoExpensiveMergeCheck()
+			throws IOException {
+		writeLooseRef("refs/heads/master", B);
+		List<ReceiveCommand> commands = Arrays.asList(
+				newCommand(B, A, "refs/heads/master",
+						ReceiveCommand.Type.UPDATE_NONFASTFORWARD));
+		BatchRefUpdate batchUpdate = refdir.newBatchUpdate();
+		batchUpdate.setAllowNonFastForwards(true);
+		batchUpdate.addCommand(commands);
+		batchUpdate.execute(new RevWalk(diskRepo) {
+			@Override
+			public boolean isMergedInto(RevCommit base, RevCommit tip) {
+				throw new AssertionError("isMergedInto() should not be called");
+			}
+		}, new StrictWorkMonitor());
+		Map<String, Ref> refs = refdir.getRefs(RefDatabase.ALL);
+		assertEquals(ReceiveCommand.Result.OK, commands.get(0).getResult());
+		assertEquals(A.getId(), refs.get("refs/heads/master").getObjectId());
+	}
+
 	@Test
 	public void testBatchRefUpdateConflict() throws IOException {
 		writeLooseRef("refs/heads/master", A);
@@ -1278,10 +1299,8 @@ public class RefDirectoryTest extends LocalDiskRepositoryTestCase {
 
 	private static ReceiveCommand newCommand(RevCommit a, RevCommit b,
 			String string, Type update) {
-		ReceiveCommand ret = new ReceiveCommand(a != null ? a.getId() : null,
+		return new ReceiveCommand(a != null ? a.getId() : null,
 				b != null ? b.getId() : null, string, update);
-		ret.setResult(ReceiveCommand.Result.NOT_ATTEMPTED);
-		return ret;
 	}
 
 	private void writeLooseRef(String name, AnyObjectId id) throws IOException {
