@@ -186,6 +186,9 @@ public class ConfigTest {
 		assertFalse(localConfig.get(UserConfig.KEY).isAuthorEmailImplicit());
 
 		// the values are defined in the global configuration
+		// first clear environment variables since they would override
+		// configuration files
+		mockSystemReader.clearProperties();
 		userGitConfig.setString("user", null, "name", "global username");
 		userGitConfig.setString("user", null, "email", "author@globalemail");
 		authorName = localConfig.get(UserConfig.KEY).getAuthorName();
@@ -211,6 +214,34 @@ public class ConfigTest {
 		assertEquals("author@localemail", authorEmail);
 		assertFalse(localConfig.get(UserConfig.KEY).isCommitterNameImplicit());
 		assertFalse(localConfig.get(UserConfig.KEY).isCommitterEmailImplicit());
+
+		// also git environment variables are defined
+		mockSystemReader.setProperty(Constants.GIT_AUTHOR_NAME_KEY,
+				"git author name");
+		mockSystemReader.setProperty(Constants.GIT_AUTHOR_EMAIL_KEY,
+				"author@email");
+		localConfig.setString("user", null, "name", "local username");
+		localConfig.setString("user", null, "email", "author@localemail");
+		authorName = localConfig.get(UserConfig.KEY).getAuthorName();
+		authorEmail = localConfig.get(UserConfig.KEY).getAuthorEmail();
+		assertEquals("git author name", authorName);
+		assertEquals("author@email", authorEmail);
+		assertFalse(localConfig.get(UserConfig.KEY).isAuthorNameImplicit());
+		assertFalse(localConfig.get(UserConfig.KEY).isAuthorEmailImplicit());
+	}
+
+	@Test
+	public void testReadUserConfigWithInvalidCharactersStripped() {
+		final MockSystemReader mockSystemReader = new MockSystemReader();
+		final Config localConfig = new Config(mockSystemReader.openUserConfig(
+				null, FS.DETECTED));
+
+		localConfig.setString("user", null, "name", "foo<bar");
+		localConfig.setString("user", null, "email", "baz>\nqux@example.com");
+
+		UserConfig userConfig = localConfig.get(UserConfig.KEY);
+		assertEquals("foobar", userConfig.getAuthorName());
+		assertEquals("bazqux@example.com", userConfig.getAuthorEmail());
 	}
 
 	@Test
