@@ -72,6 +72,8 @@ import org.eclipse.jgit.errors.CheckoutConflictException;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.NoWorkTreeException;
 import org.eclipse.jgit.junit.RepositoryTestCase;
+import org.eclipse.jgit.junit.TestRepository;
+import org.eclipse.jgit.junit.TestRepository.BranchBuilder;
 import org.eclipse.jgit.revwalk.RevCommit;
 import org.eclipse.jgit.treewalk.FileTreeIterator;
 import org.eclipse.jgit.treewalk.TreeWalk;
@@ -213,6 +215,23 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 		assertIndex(mkmap("x", "x"));
 	}
 
+	/**
+	 * Test first checkout in a repo
+	 *
+	 * @throws Exception
+	 */
+	@Test
+	public void testInitialCheckout() throws Exception {
+		Git git = new Git(db);
+
+		TestRepository<Repository> db_t = new TestRepository<Repository>(db);
+		BranchBuilder master = db_t.branch("master");
+		master.commit().add("f", "1").message("m0").create();
+		assertFalse(new File(db.getWorkTree(), "f").exists());
+		git.checkout().setName("master").call();
+		assertTrue(new File(db.getWorkTree(), "f").exists());
+	}
+
 	private DirCacheCheckout resetHard(RevCommit commit)
 			throws NoWorkTreeException,
 			CorruptObjectException, IOException {
@@ -619,7 +638,7 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 	@Test
 	public void testDirectoryFileConflicts_9() throws Exception {
 		// 9
-		doit(mk("DF"), mkmap("DF", "QP"), mk("DF/DF"));
+		doit(mkmap("DF", "QP"), mkmap("DF", "QP"), mkmap("DF/DF", "DF/DF"));
 		assertRemoved("DF/DF");
 		assertUpdated("DF");
 	}
@@ -966,6 +985,38 @@ public class DirCacheCheckoutTest extends RepositoryTestCase {
 		}
 	}
 
+	@Test
+	public void testOverwriteUntrackedIgnoredFile() throws IOException,
+			GitAPIException {
+		String fname="file.txt";
+		Git git = Git.wrap(db);
+
+		// Add a file
+		writeTrashFile(fname, "a");
+		git.add().addFilepattern(fname).call();
+		git.commit().setMessage("create file").call();
+
+		// Create branch
+		git.branchCreate().setName("side").call();
+
+		// Modify file
+		writeTrashFile(fname, "b");
+		git.add().addFilepattern(fname).call();
+		git.commit().setMessage("modify file").call();
+
+		// Switch branches
+		git.checkout().setName("side").call();
+		git.rm().addFilepattern(fname).call();
+		writeTrashFile(".gitignore", fname);
+		git.add().addFilepattern(".gitignore").call();
+		git.commit().setMessage("delete and ignore file").call();
+
+		writeTrashFile(fname, "Something different");
+		git.checkout().setName("master").call();
+		assertWorkDir(mkmap(fname, "b"));
+		assertTrue(git.status().call().isClean());
+	}
+
 	@Test
 	public void testFileModeChangeWithNoContentChangeUpdate() throws Exception {
 		if (!FS.DETECTED.supportsExecute())
