@@ -68,9 +68,11 @@ import org.eclipse.jgit.dircache.DirCacheEditor;
 import org.eclipse.jgit.dircache.DirCacheEditor.PathEdit;
 import org.eclipse.jgit.dircache.DirCacheEntry;
 import org.eclipse.jgit.junit.RepositoryTestCase;
+import org.eclipse.jgit.lib.CoreConfig.AutoCRLF;
 import org.eclipse.jgit.lib.IndexDiff.StageState;
 import org.eclipse.jgit.merge.MergeStrategy;
 import org.eclipse.jgit.revwalk.RevCommit;
+import org.eclipse.jgit.storage.file.FileBasedConfig;
 import org.eclipse.jgit.treewalk.FileTreeIterator;
 import org.eclipse.jgit.util.IO;
 import org.junit.Test;
@@ -467,6 +469,53 @@ public class IndexDiffTest extends RepositoryTestCase {
 				diff.getUntrackedFolders());
 	}
 
+	/**
+	 * Test that ignored folders aren't listed as untracked
+	 *
+	 * @throws Exception
+	 */
+	@Test
+	public void testUntrackedNotIgnoredFolders() throws Exception {
+		Git git = new Git(db);
+
+		IndexDiff diff = new IndexDiff(db, Constants.HEAD,
+				new FileTreeIterator(db));
+		diff.diff();
+		assertEquals(Collections.EMPTY_SET, diff.getUntrackedFolders());
+
+		writeTrashFile("readme", "");
+		writeTrashFile("sr/com/X.java", "");
+		writeTrashFile("src/com/A.java", "");
+		writeTrashFile("src/org/B.java", "");
+		writeTrashFile("srcs/org/Y.java", "");
+		writeTrashFile("target/com/A.java", "");
+		writeTrashFile("target/org/B.java", "");
+		writeTrashFile(".gitignore", "/target\n/sr");
+
+		git.add().addFilepattern("readme").addFilepattern(".gitignore")
+				.addFilepattern("srcs/").call();
+		git.commit().setMessage("initial").call();
+
+		diff = new IndexDiff(db, Constants.HEAD, new FileTreeIterator(db));
+		diff.diff();
+		assertEquals(new HashSet<String>(Arrays.asList("src")),
+				diff.getUntrackedFolders());
+
+		git.add().addFilepattern("src").call();
+		writeTrashFile("sr/com/X1.java", "");
+		writeTrashFile("src/tst/A.java", "");
+		writeTrashFile("src/tst/B.java", "");
+		writeTrashFile("srcs/com/Y1.java", "");
+		deleteTrashFile(".gitignore");
+
+		diff = new IndexDiff(db, Constants.HEAD, new FileTreeIterator(db));
+		diff.diff();
+		assertEquals(
+				new HashSet<String>(Arrays.asList("srcs/com", "sr", "src/tst",
+						"target")),
+				diff.getUntrackedFolders());
+	}
+
 	@Test
 	public void testAssumeUnchanged() throws Exception {
 		Git git = new Git(db);
@@ -529,6 +578,151 @@ public class IndexDiffTest extends RepositoryTestCase {
 		assertTrue(StageState.BOTH_ADDED.hasTheirs());
 	}
 
+	@Test
+	public void testStageState_mergeAndReset_bug() throws Exception {
+		Git git = new Git(db);
+
+		writeTrashFile("a", "content");
+		git.add().addFilepattern("a").call();
+		RevCommit initialCommit = git.commit().setMessage("initial commit")
+				.call();
+
+		// create branch and add a new file
+		final String branchName = Constants.R_HEADS + "branch";
+		createBranch(initialCommit, branchName);
+		checkoutBranch(branchName);
+		writeTrashFile("b", "second file content - branch");
+		git.add().addFilepattern("b").call();
+		RevCommit branchCommit = git.commit().setMessage("branch commit")
+				.call();
+
+		// checkout master and add the same new file
+		checkoutBranch(Constants.R_HEADS + Constants.MASTER);
+		writeTrashFile("b", "second file content - master");
+		git.add().addFilepattern("b").call();
+		git.commit().setMessage("master commit").call();
+
+		// try and merge
+		MergeResult result = git.merge().include(branchCommit).call();
+		assertEquals(MergeStatus.CONFLICTING, result.getMergeStatus());
+
+		FileTreeIterator iterator = new FileTreeIterator(db);
+		IndexDiff diff = new IndexDiff(db, Constants.HEAD, iterator);
+		diff.diff();
+
+		assertTrue(diff.getChanged().isEmpty());
+		assertTrue(diff.getAdded().isEmpty());
+		assertTrue(diff.getRemoved().isEmpty());
+		assertTrue(diff.getMissing().isEmpty());
+		assertTrue(diff.getModified().isEmpty());
+		assertEquals(1, diff.getConflicting().size());
+		assertTrue(diff.getConflicting().contains("b"));
+		assertEquals(StageState.BOTH_ADDED, diff.getConflictingStageStates()
+				.get("b"));
+		assertTrue(diff.getUntrackedFolders().isEmpty());
+
+		// reset file b to its master state without altering the index
+		writeTrashFile("b", "second file content - master");
+
+		// we should have the same result
+		iterator = new FileTreeIterator(db);
+		diff = new IndexDiff(db, Constants.HEAD, iterator);
+		diff.diff();
+
+		assertTrue(diff.getChanged().isEmpty());
+		assertTrue(diff.getAdded().isEmpty());
+		assertTrue(diff.getRemoved().isEmpty());
+		assertTrue(diff.getMissing().isEmpty());
+		assertTrue(diff.getModified().isEmpty());
+		assertEquals(1, diff.getConflicting().size());
+		assertTrue(diff.getConflicting().contains("b"));
+		assertEquals(StageState.BOTH_ADDED, diff.getConflictingStageStates()
+				.get("b"));
+		assertTrue(diff.getUntrackedFolders().isEmpty());
+	}
+
+	@Test
+	public void testStageState_simulated_bug() throws Exception {
+		Git git = new Git(db);
+
+		writeTrashFile("a", "content");
+		git.add().addFilepattern("a").call();
+		RevCommit initialCommit = git.commit().setMessage("initial commit")
+				.call();
+
+		// create branch and add a new file
+		final String branchName = Constants.R_HEADS + "branch";
+		createBranch(initialCommit, branchName);
+		checkoutBranch(branchName);
+		writeTrashFile("b", "second file content - branch");
+		git.add().addFilepattern("b").call();
+		git.commit().setMessage("branch commit")
+				.call();
+
+		// checkout master and add the same new file
+		checkoutBranch(Constants.R_HEADS + Constants.MASTER);
+		writeTrashFile("b", "second file content - master");
+		git.add().addFilepattern("b").call();
+		git.commit().setMessage("master commit").call();
+
+		// Simulate a failed merge of branch into master
+		DirCacheBuilder builder = db.lockDirCache().builder();
+		DirCacheEntry entry = createEntry("a", FileMode.REGULAR_FILE, 0,
+				"content");
+		builder.add(entry);
+		entry = createEntry("b", FileMode.REGULAR_FILE, 2,
+				"second file content - master");
+		builder.add(entry);
+		entry = createEntry("b", FileMode.REGULAR_FILE, 3,
+				"second file content - branch");
+		builder.add(entry);
+		builder.commit();
+
+		FileTreeIterator iterator = new FileTreeIterator(db);
+		IndexDiff diff = new IndexDiff(db, Constants.HEAD, iterator);
+		diff.diff();
+
+		assertTrue(diff.getChanged().isEmpty());
+		assertTrue(diff.getAdded().isEmpty());
+		assertTrue(diff.getRemoved().isEmpty());
+		assertTrue(diff.getMissing().isEmpty());
+		assertTrue(diff.getModified().isEmpty());
+		assertEquals(1, diff.getConflicting().size());
+		assertTrue(diff.getConflicting().contains("b"));
+		assertEquals(StageState.BOTH_ADDED, diff.getConflictingStageStates()
+				.get("b"));
+		assertTrue(diff.getUntrackedFolders().isEmpty());
+	}
+
+	@Test
+	public void testAutoCRLFInput() throws Exception {
+		Git git = new Git(db);
+		FileBasedConfig config = db.getConfig();
+
+		// Make sure core.autocrlf is false before adding
+		config.setEnum(ConfigConstants.CONFIG_CORE_SECTION, null,
+				ConfigConstants.CONFIG_KEY_AUTOCRLF, AutoCRLF.FALSE);
+		config.save();
+
+		// File is already in repository with CRLF
+		writeTrashFile("crlf.txt", "this\r\ncontains\r\ncrlf\r\n");
+		git.add().addFilepattern("crlf.txt").call();
+		git.commit().setMessage("Add crlf.txt").call();
+
+		// Now set core.autocrlf to input
+		config.setEnum(ConfigConstants.CONFIG_CORE_SECTION, null,
+				ConfigConstants.CONFIG_KEY_AUTOCRLF, AutoCRLF.INPUT);
+		config.save();
+
+		FileTreeIterator iterator = new FileTreeIterator(db);
+		IndexDiff diff = new IndexDiff(db, Constants.HEAD, iterator);
+		diff.diff();
+
+		assertTrue(
+				"Expected no modified files, but there were: "
+						+ diff.getModified(), diff.getModified().isEmpty());
+	}
+
 	private void verifyStageState(StageState expected, int... stages)
 			throws IOException {
 		DirCacheBuilder builder = db.lockDirCache().builder();
