@@ -43,7 +43,6 @@
 package org.eclipse.jgit.merge;
 
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 
 import java.io.File;
@@ -54,6 +53,7 @@ import org.eclipse.jgit.api.Git;
 import org.eclipse.jgit.api.MergeResult;
 import org.eclipse.jgit.api.MergeResult.MergeStatus;
 import org.eclipse.jgit.api.errors.CheckoutConflictException;
+import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.api.errors.JGitInternalException;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.errors.NoMergeBaseException;
@@ -79,7 +79,7 @@ public class ResolveMergerTest extends RepositoryTestCase {
 	public static MergeStrategy recursive = MergeStrategy.RECURSIVE;
 
 	@Theory
-	public void failingPathsShouldNotResultInOKReturnValue(
+	public void failingDeleteOfDirectoryWithUntrackedContent(
 			MergeStrategy strategy) throws Exception {
 		File folder1 = new File(db.getWorkTree(), "folder1");
 		FileUtils.mkdir(folder1);
@@ -107,6 +107,7 @@ public class ResolveMergerTest extends RepositoryTestCase {
 		RevCommit head = git.commit().setMessage("Adding another file").call();
 
 		// Untracked file to cause failing path for delete() of folder1
+		// but that's ok.
 		file = new File(folder1, "file3.txt");
 		write(file, "folder1--file3.txt");
 
@@ -114,9 +115,8 @@ public class ResolveMergerTest extends RepositoryTestCase {
 		merger.setCommitNames(new String[] { "BASE", "HEAD", "other" });
 		merger.setWorkingTreeIterator(new FileTreeIterator(db));
 		boolean ok = merger.merge(head.getId(), other.getId());
-
-		assertFalse(merger.getFailingPaths().isEmpty());
-		assertFalse(ok);
+		assertTrue(ok);
+		assertTrue(file.exists());
 	}
 
 	/**
@@ -265,6 +265,37 @@ public class ResolveMergerTest extends RepositoryTestCase {
 				indexState(CONTENT));
 	}
 
+	@Theory
+	public void mergeWithCrlfInWT(MergeStrategy strategy) throws IOException,
+			GitAPIException {
+		Git git = Git.wrap(db);
+		db.getConfig().setString("core", null, "autocrlf", "false");
+		db.getConfig().save();
+		writeTrashFile("crlf.txt", "some\r\ndata\r\n");
+		git.add().addFilepattern("crlf.txt").call();
+		git.commit().setMessage("base").call();
+
+		git.branchCreate().setName("brancha").call();
+
+		writeTrashFile("crlf.txt", "some\r\nmore\r\ndata\r\n");
+		git.add().addFilepattern("crlf.txt").call();
+		git.commit().setMessage("on master").call();
+
+		git.checkout().setName("brancha").call();
+		writeTrashFile("crlf.txt", "some\r\ndata\r\ntoo\r\n");
+		git.add().addFilepattern("crlf.txt").call();
+		git.commit().setMessage("on brancha").call();
+
+		db.getConfig().setString("core", null, "autocrlf", "input");
+		db.getConfig().save();
+
+		MergeResult mergeResult = git.merge().setStrategy(strategy)
+				.include(db.resolve("master"))
+				.call();
+		assertEquals(MergeResult.MergeStatus.MERGED,
+				mergeResult.getMergeStatus());
+	}
+
 	/**
 	 * Merging two equal subtrees when the index does not contain any file in
 	 * that subtree should lead to a merged state.
