@@ -56,11 +56,15 @@ import java.io.File;
 import java.io.FileWriter;
 import java.io.IOException;
 
+import org.eclipse.jgit.api.Git;
+import org.eclipse.jgit.api.Status;
+import org.eclipse.jgit.api.errors.GitAPIException;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.dircache.DirCacheEditor;
 import org.eclipse.jgit.dircache.DirCacheEditor.PathEdit;
 import org.eclipse.jgit.dircache.DirCacheEntry;
 import org.eclipse.jgit.errors.ConfigInvalidException;
+import org.eclipse.jgit.errors.NoWorkTreeException;
 import org.eclipse.jgit.junit.RepositoryTestCase;
 import org.eclipse.jgit.junit.TestRepository;
 import org.eclipse.jgit.lib.Config;
@@ -98,7 +102,7 @@ public class SubmoduleWalkTest extends RepositoryTestCase {
 
 	@Test
 	public void repositoryWithRootLevelSubmodule() throws IOException,
-			ConfigInvalidException {
+			ConfigInvalidException, NoWorkTreeException, GitAPIException {
 		final ObjectId id = ObjectId
 				.fromString("abcd1234abcd1234abcd1234abcd1234abcd1234");
 		final String path = "sub";
@@ -124,6 +128,8 @@ public class SubmoduleWalkTest extends RepositoryTestCase {
 		assertNull(gen.getModulesUpdate());
 		assertNull(gen.getModulesUrl());
 		assertNull(gen.getRepository());
+		Status status = Git.wrap(db).status().call();
+		assertTrue(!status.isClean());
 		assertFalse(gen.next());
 	}
 
@@ -169,12 +175,12 @@ public class SubmoduleWalkTest extends RepositoryTestCase {
 		assertNull(gen.getModulesUpdate());
 		assertNull(gen.getModulesUrl());
 		Repository subRepo = gen.getRepository();
-		addRepoToClose(subRepo);
 		assertNotNull(subRepo);
 		assertEquals(modulesGitDir.getAbsolutePath(),
 				subRepo.getDirectory().getAbsolutePath());
 		assertEquals(new File(db.getWorkTree(), path).getAbsolutePath(),
 				subRepo.getWorkTree().getAbsolutePath());
+		subRepo.close();
 		assertFalse(gen.next());
 	}
 
@@ -221,11 +227,11 @@ public class SubmoduleWalkTest extends RepositoryTestCase {
 		assertNull(gen.getModulesUpdate());
 		assertNull(gen.getModulesUrl());
 		Repository subRepo = gen.getRepository();
-		addRepoToClose(subRepo);
 		assertNotNull(subRepo);
 		assertEqualsFile(modulesGitDir, subRepo.getDirectory());
 		assertEqualsFile(new File(db.getWorkTree(), path),
 				subRepo.getWorkTree());
+		subRepo.close();
 		assertFalse(gen.next());
 	}
 
