@@ -144,6 +144,43 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		assertTrue(fti.eof());
 	}
 
+	@Test
+	public void testEmptyIteratorOnEmptyDirectory() throws Exception {
+		String nonExistingFileName = "not-existing-file";
+		final File r = new File(trash, nonExistingFileName);
+		assertFalse(r.exists());
+		FileUtils.mkdir(r);
+
+		final FileTreeIterator parent = new FileTreeIterator(db);
+
+		while (!parent.getEntryPathString().equals(nonExistingFileName))
+			parent.next(1);
+
+		final FileTreeIterator childIter = new FileTreeIterator(parent, r,
+				db.getFS());
+		assertTrue(childIter.first());
+		assertTrue(childIter.eof());
+
+		String parentPath = parent.getEntryPathString();
+		assertEquals(nonExistingFileName, parentPath);
+
+		// must be "not-existing-file/", but getEntryPathString() was broken by
+		// 445363 too
+		String childPath = childIter.getEntryPathString();
+
+		// in bug 445363 the iterator wrote garbage to the parent "path" field
+		EmptyTreeIterator e = childIter.createEmptyTreeIterator();
+		assertNotNull(e);
+
+		// check if parent path is not overridden by empty iterator (bug 445363)
+		// due bug 445363 this was "/ot-existing-file" instead of
+		// "not-existing-file"
+		assertEquals(parentPath, parent.getEntryPathString());
+		assertEquals(parentPath + "/", childPath);
+		assertEquals(parentPath + "/", childIter.getEntryPathString());
+		assertEquals(childPath + "/", e.getEntryPathString());
+	}
+
 	@Test
 	public void testSimpleIterate() throws Exception {
 		final FileTreeIterator top = new FileTreeIterator(trash, db.getFS(),
@@ -244,7 +281,7 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 
 	@Test
 	public void testIsModifiedSymlinkAsFile() throws Exception {
-		File f = writeTrashFile("symlink", "content");
+		writeTrashFile("symlink", "content");
 		Git git = new Git(db);
 		db.getConfig().setString(ConfigConstants.CONFIG_CORE_SECTION, null,
 				ConfigConstants.CONFIG_KEY_SYMLINKS, "false");
@@ -254,13 +291,13 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		// Modify previously committed DirCacheEntry and write it back to disk
 		DirCacheEntry dce = db.readDirCache().getEntry("symlink");
 		dce.setFileMode(FileMode.SYMLINK);
-		DirCacheCheckout.checkoutEntry(db, f, dce);
+		ObjectReader objectReader = db.newObjectReader();
+		DirCacheCheckout.checkoutEntry(db, dce, objectReader);
 
 		FileTreeIterator fti = new FileTreeIterator(trash, db.getFS(), db
 				.getConfig().get(WorkingTreeOptions.KEY));
 		while (!fti.getEntryPathString().equals("symlink"))
 			fti.next(1);
-		ObjectReader objectReader = db.newObjectReader();
 		assertFalse(fti.isModified(dce, false, objectReader));
 		objectReader.release();
 	}
@@ -277,6 +314,11 @@ public class FileTreeIteratorTest extends RepositoryTestCase {
 		git.add().addFilepattern("file").call();
 		writeTrashFile("file", "conten2");
 		f.setLastModified(lastModified);
+		// We cannot trust this to go fast enough on
+		// a system with less than one-second lastModified
+		// resolution, so we force the index to have the
+		// same timestamp as the file we look at.
+		db.getIndexFile().setLastModified(lastModified);
 		DirCacheEntry dce = db.readDirCache().getEntry("file");
 		FileTreeIterator fti = new FileTreeIterator(trash, db.getFS(), db
 				.getConfig().get(WorkingTreeOptions.KEY));
