@@ -639,6 +639,10 @@ public class ChangeIdUtilTest {
 
 	@Test
 	public void testIndexOfChangeId() {
+		assertEquals(-1, ChangeIdUtil.indexOfChangeId("", "\n"));
+		assertEquals(-1, ChangeIdUtil.indexOfChangeId("\n", "\n"));
+		assertEquals(-1, ChangeIdUtil.indexOfChangeId("\r\n", "\r\n"));
+
 		assertEquals(3, ChangeIdUtil.indexOfChangeId("x\n" + "\n"
 				+ "Change-Id: I3b7e4e16b503ce00f07ba6ad01d97a356dad7701\n",
 				"\n"));
