@@ -43,7 +43,6 @@
  */
 package org.eclipse.jgit.api;
 
-import java.io.File;
 import java.io.IOException;
 import java.text.MessageFormat;
 import java.util.ArrayList;
@@ -81,7 +80,6 @@ import org.eclipse.jgit.revwalk.RevTree;
 import org.eclipse.jgit.revwalk.RevWalk;
 import org.eclipse.jgit.treewalk.TreeWalk;
 import org.eclipse.jgit.treewalk.filter.PathFilterGroup;
-import org.eclipse.jgit.util.FileUtils;
 
 /**
  * Checkout a branch to the working tree.
@@ -213,7 +211,10 @@ public class CheckoutCommand extends GitCommand<Ref> {
 				Git git = new Git(repo);
 				CreateBranchCommand command = git.branchCreate();
 				command.setName(name);
-				command.setStartPoint(getStartPoint().name());
+				if (startCommit != null)
+					command.setStartPoint(startCommit);
+				else
+					command.setStartPoint(startPoint);
 				if (upstreamMode != null)
 					command.setUpstreamMode(upstreamMode);
 				command.call();
@@ -234,7 +235,7 @@ public class CheckoutCommand extends GitCommand<Ref> {
 					this.status = CheckoutResult.NOT_TRIED_RESULT;
 					return repo.getRef(Constants.HEAD);
 				}
-				branch = getStartPoint();
+				branch = getStartPointObjectId();
 			} else {
 				branch = repo.resolve(name);
 				if (branch == null)
@@ -386,7 +387,7 @@ public class CheckoutCommand extends GitCommand<Ref> {
 				if (isCheckoutIndex())
 					checkoutPathsFromIndex(treeWalk, dc);
 				else {
-					RevCommit commit = revWalk.parseCommit(getStartPoint());
+					RevCommit commit = revWalk.parseCommit(getStartPointObjectId());
 					checkoutPathsFromCommit(treeWalk, dc, commit);
 				}
 			} finally {
@@ -404,14 +405,17 @@ public class CheckoutCommand extends GitCommand<Ref> {
 		DirCacheIterator dci = new DirCacheIterator(dc);
 		treeWalk.addTree(dci);
 
+		String previousPath = null;
+
 		final ObjectReader r = treeWalk.getObjectReader();
 		DirCacheEditor editor = dc.editor();
 		while (treeWalk.next()) {
-			DirCacheEntry entry = dci.getDirCacheEntry();
+			String path = treeWalk.getPathString();
 			// Only add one edit per path
-			if (entry != null && entry.getStage() > DirCacheEntry.STAGE_1)
+			if (path.equals(previousPath))
 				continue;
-			editor.add(new PathEdit(treeWalk.getPathString()) {
+
+			editor.add(new PathEdit(path) {
 				public void apply(DirCacheEntry ent) {
 					int stage = ent.getStage();
 					if (stage > DirCacheEntry.STAGE_0) {
@@ -428,6 +432,8 @@ public class CheckoutCommand extends GitCommand<Ref> {
 					}
 				}
 			});
+
+			previousPath = path;
 		}
 		editor.commit();
 	}
@@ -452,11 +458,8 @@ public class CheckoutCommand extends GitCommand<Ref> {
 	}
 
 	private void checkoutPath(DirCacheEntry entry, ObjectReader reader) {
-		File file = new File(repo.getWorkTree(), entry.getPathString());
-		File parentDir = file.getParentFile();
 		try {
-			FileUtils.mkdirs(parentDir, true);
-			DirCacheCheckout.checkoutEntry(repo, file, entry, reader);
+			DirCacheCheckout.checkoutEntry(repo, entry, reader);
 		} catch (IOException e) {
 			throw new JGitInternalException(MessageFormat.format(
 					JGitText.get().checkoutConflictWithFile,
@@ -468,21 +471,17 @@ public class CheckoutCommand extends GitCommand<Ref> {
 		return startCommit == null && startPoint == null;
 	}
 
-	private ObjectId getStartPoint() throws AmbiguousObjectException,
+	private ObjectId getStartPointObjectId() throws AmbiguousObjectException,
 			RefNotFoundException, IOException {
 		if (startCommit != null)
 			return startCommit.getId();
-		ObjectId result = null;
-		try {
-			result = repo.resolve((startPoint == null) ? Constants.HEAD
-					: startPoint);
-		} catch (AmbiguousObjectException e) {
-			throw e;
-		}
+
+		String startPointOrHead = (startPoint != null) ? startPoint
+				: Constants.HEAD;
+		ObjectId result = repo.resolve(startPointOrHead);
 		if (result == null)
 			throw new RefNotFoundException(MessageFormat.format(
-					JGitText.get().refNotResolved,
-					startPoint != null ? startPoint : Constants.HEAD));
+					JGitText.get().refNotResolved, startPointOrHead));
 		return result;
 	}
 
