@@ -556,7 +556,8 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 			pin(pack, 0);
 			if (block.copy(0, buf, 0, 12) != 12) {
 				pack.setInvalid();
-				throw new IOException(JGitText.get().packfileIsTruncated);
+				throw new IOException(MessageFormat.format(
+						JGitText.get().packfileIsTruncated, pack.getPackName()));
 			}
 			md.update(buf, 0, 12);
 		}
@@ -580,7 +581,8 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 			pin(pack, position);
 			if (block.copy(position, buf, 0, 20) != 20) {
 				pack.setInvalid();
-				throw new IOException(JGitText.get().packfileIsTruncated);
+				throw new IOException(MessageFormat.format(
+						JGitText.get().packfileIsTruncated, pack.getPackName()));
 			}
 			if (!Arrays.equals(actHash, buf)) {
 				pack.setInvalid();
@@ -600,11 +602,11 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 	 *            position within the file to read from.
 	 * @param dstbuf
 	 *            destination buffer the inflater should output decompressed
-	 *            data to.
+	 *            data to. Must be large enough to store the entire stream,
+	 *            unless headerOnly is true.
 	 * @param headerOnly
 	 *            if true the caller wants only {@code dstbuf.length} bytes.
-	 * @return updated <code>dstoff</code> based on the number of bytes
-	 *         successfully inflated into <code>dstbuf</code>.
+	 * @return number of bytes inflated into <code>dstbuf</code>.
 	 * @throws IOException
 	 *             this cursor does not match the provider or id and the proper
 	 *             window could not be acquired through the provider's cache.
@@ -616,24 +618,17 @@ public final class DfsReader extends ObjectReader implements ObjectReuseAsIs {
 			boolean headerOnly) throws IOException, DataFormatException {
 		prepareInflater();
 		pin(pack, position);
-		block.setInput(inf, position);
-		int dstoff = 0;
-		for (;;) {
+		position += block.setInput(position, inf);
+		for (int dstoff = 0;;) {
 			int n = inf.inflate(dstbuf, dstoff, dstbuf.length - dstoff);
-			if (n == 0) {
-				if (headerOnly && dstoff == dstbuf.length)
-					return dstoff;
-				if (inf.needsInput()) {
-					position += block.remaining(position);
-					pin(pack, position);
-					block.setInput(inf, position);
-					continue;
-				}
-				if (inf.finished())
-					return dstoff;
-				throw new DataFormatException();
-			}
 			dstoff += n;
+			if (inf.finished() || (headerOnly && dstoff == dstbuf.length))
+				return dstoff;
+			if (inf.needsInput()) {
+				pin(pack, position);
+				position += block.setInput(position, inf);
+			} else if (n == 0)
+				throw new DataFormatException();
 		}
 	}
 
