@@ -65,6 +65,7 @@ import java.util.zip.DataFormatException;
 import java.util.zip.Inflater;
 
 import org.eclipse.jgit.errors.CorruptObjectException;
+import org.eclipse.jgit.errors.LargeObjectException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.PackInvalidException;
 import org.eclipse.jgit.errors.PackMismatchException;
@@ -168,11 +169,13 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 			try {
 				final PackIndex idx = PackIndex.open(extFile(INDEX));
 
-				if (packChecksum == null)
+				if (packChecksum == null) {
 					packChecksum = idx.packChecksum;
-				else if (!Arrays.equals(packChecksum, idx.packChecksum))
-					throw new PackMismatchException(JGitText.get().packChecksumMismatch);
-
+				} else if (!Arrays.equals(packChecksum, idx.packChecksum)) {
+					throw new PackMismatchException(MessageFormat.format(
+							JGitText.get().packChecksumMismatch,
+							packFile.getPath()));
+				}
 				loadedIdx = idx;
 			} catch (IOException e) {
 				invalid = true;
@@ -334,7 +337,7 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 			return null;
 		}
 
-		if (curs.inflate(this, position, dstbuf, 0) != sz)
+		if (curs.inflate(this, position, dstbuf, false) != sz)
 			throw new EOFException(MessageFormat.format(
 					JGitText.get().shortCompressedStreamAt,
 					Long.valueOf(position)));
@@ -701,7 +704,7 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 	}
 
 	ObjectLoader load(final WindowCursor curs, long pos)
-			throws IOException {
+			throws IOException, LargeObjectException {
 		try {
 			final byte[] ib = curs.tempId;
 			Delta delta = null;
@@ -727,7 +730,7 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 				case Constants.OBJ_TREE:
 				case Constants.OBJ_BLOB:
 				case Constants.OBJ_TAG: {
-					if (sz < curs.getStreamFileThreshold())
+					if (delta != null || sz < curs.getStreamFileThreshold())
 						data = decompress(pos + p, (int) sz, curs);
 
 					if (delta != null) {
@@ -796,7 +799,7 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 			// (Whole objects with no deltas to apply return early above.)
 
 			if (data == null)
-				return delta.large(this, curs);
+				throw new IOException(JGitText.get().inMemoryBufferLimitExceeded);
 
 			do {
 				// Cache only the base immediately before desired object.
@@ -811,19 +814,19 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 						delta.deltaSize, curs);
 				if (cmds == null) {
 					data = null; // Discard base in case of OutOfMemoryError
-					return delta.large(this, curs);
+					throw new LargeObjectException.OutOfMemory(new OutOfMemoryError());
 				}
 
 				final long sz = BinaryDelta.getResultSize(cmds);
 				if (Integer.MAX_VALUE <= sz)
-					return delta.large(this, curs);
+					throw new LargeObjectException.ExceedsByteArrayLimit();
 
 				final byte[] result;
 				try {
 					result = new byte[(int) sz];
 				} catch (OutOfMemoryError tooBig) {
 					data = null; // Discard base in case of OutOfMemoryError
-					return delta.large(this, curs);
+					throw new LargeObjectException.OutOfMemory(tooBig);
 				}
 
 				BinaryDelta.apply(data, cmds, result);
@@ -875,18 +878,6 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 			this.hdrLen = hdrLen;
 			this.basePos = baseOffset;
 		}
-
-		ObjectLoader large(PackFile pack, WindowCursor wc) {
-			Delta d = this;
-			while (d.next != null)
-				d = d.next;
-			return d.newLargeLoader(pack, wc);
-		}
-
-		private ObjectLoader newLargeLoader(PackFile pack, WindowCursor wc) {
-			return new LargePackedDeltaObject(deltaPos, basePos, hdrLen,
-					pack, wc.db);
-		}
 	}
 
 	byte[] getDeltaHeader(WindowCursor wc, long pos)
@@ -897,7 +888,7 @@ public class PackFile implements Iterable<PackIndex.MutableEntry> {
 		// the longest delta instruction header.
 		//
 		final byte[] hdr = new byte[18];
-		wc.inflate(this, pos, hdr, 0);
+		wc.inflate(this, pos, hdr, true /* headerOnly */);
 		return hdr;
 	}
 
