@@ -79,7 +79,6 @@ public class ConfigConstants {
 
 	/**
 	 * The "rebase" section
-	 *
 	 * @since 3.2
 	 */
 	public static final String CONFIG_REBASE_SECTION = "rebase";
@@ -92,11 +91,16 @@ public class ConfigConstants {
 
 	/**
 	 * The "fetch" section
-	 *
 	 * @since 3.3
 	 */
 	public static final String CONFIG_FETCH_SECTION = "fetch";
 
+	/**
+	 * The "pull" section
+	 * @since 3.5
+	 */
+	public static final String CONFIG_PULL_SECTION = "pull";
+
 	/** The "algorithm" key */
 	public static final String CONFIG_KEY_ALGORITHM = "algorithm";
 
@@ -109,6 +113,13 @@ public class ConfigConstants {
 	/** The "excludesfile" key */
 	public static final String CONFIG_KEY_EXCLUDESFILE = "excludesfile";
 
+	/**
+	 * The "attributesfile" key
+	 *
+	 * @since 3.7
+	 */
+	public static final String CONFIG_KEY_ATTRIBUTESFILE = "attributesfile";
+
 	/** The "filemode" key */
 	public static final String CONFIG_KEY_FILEMODE = "filemode";
 
@@ -132,7 +143,6 @@ public class ConfigConstants {
 
 	/**
 	 * The "symlinks" key
-	 *
 	 * @since 3.3
 	 */
 	public static final String CONFIG_KEY_SYMLINKS = "symlinks";
@@ -160,7 +170,6 @@ public class ConfigConstants {
 
 	/**
 	 * The "autostash" key
-	 *
 	 * @since 3.2
 	 */
 	public static final String CONFIG_KEY_AUTOSTASH = "autostash";
@@ -201,18 +210,42 @@ public class ConfigConstants {
 	/** The "update" key */
 	public static final String CONFIG_KEY_UPDATE = "update";
 
+	/**
+	 * The "ignore" key
+	 * @since 3.6
+	 */
+	public static final String CONFIG_KEY_IGNORE = "ignore";
+
 	/** The "compression" key */
 	public static final String CONFIG_KEY_COMPRESSION = "compression";
 
 	/** The "indexversion" key */
 	public static final String CONFIG_KEY_INDEXVERSION = "indexversion";
 
+	/**
+	 * The "hidedotfiles" key
+	 * @since 3.5
+	 */
+	public static final String CONFIG_KEY_HIDEDOTFILES = "hidedotfiles";
+
 	/** The "precomposeunicode" key */
 	public static final String CONFIG_KEY_PRECOMPOSEUNICODE = "precomposeunicode";
 
 	/** The "pruneexpire" key */
 	public static final String CONFIG_KEY_PRUNEEXPIRE = "pruneexpire";
 
+	/**
+	 * The "aggressiveDepth" key
+	 * @since 3.6
+	 */
+	public static final String CONFIG_KEY_AGGRESSIVE_DEPTH = "aggressiveDepth";
+
+	/**
+	 * The "aggressiveWindow" key
+	 * @since 3.6
+	 */
+	public static final String CONFIG_KEY_AGGRESSIVE_WINDOW = "aggressiveWindow";
+
 	/** The "mergeoptions" key */
 	public static final String CONFIG_KEY_MERGEOPTIONS = "mergeoptions";
 
@@ -226,38 +259,43 @@ public class ConfigConstants {
 	public static final String CONFIG_KEY_CHECKSTAT = "checkstat";
 
 	/**
-         * The "renamelimit" key in the "diff section"
-         * @since 3.0
-         */
+	 * The "renamelimit" key in the "diff section"
+	 * @since 3.0
+	 */
 	public static final String CONFIG_KEY_RENAMELIMIT = "renamelimit";
 
 	/**
-         * The "noprefix" key in the "diff section"
-         * @since 3.0
-         */
+	 * The "trustfolderstat" key in the "core section"
+	 * @since 3.6
+	 */
+	public static final String CONFIG_KEY_TRUSTFOLDERSTAT = "trustfolderstat";
+
+	/**
+	 * The "noprefix" key in the "diff section"
+	 * @since 3.0
+	 */
 	public static final String CONFIG_KEY_NOPREFIX = "noprefix";
 
 	/**
-         * A "renamelimit" value in the "diff section"
-         * @since 3.0
-         */
+	 * A "renamelimit" value in the "diff section"
+	 * @since 3.0
+	 */
 	public static final String CONFIG_RENAMELIMIT_COPY = "copy";
 
 	/**
-         * A "renamelimit" value in the "diff section"
-         * @since 3.0
-         */
+	 * A "renamelimit" value in the "diff section"
+	 * @since 3.0
+	 */
 	public static final String CONFIG_RENAMELIMIT_COPIES = "copies";
 
 	/**
-         * The "renames" key in the "diff section"
-         * @since 3.0
-         */
+	 * The "renames" key in the "diff section"
+	 * @since 3.0
+	 */
 	public static final String CONFIG_KEY_RENAMES = "renames";
 
 	/**
 	 * The "prune" key
-	 *
 	 * @since 3.3
 	 */
 	public static final String CONFIG_KEY_PRUNE = "prune";
