@@ -65,8 +65,6 @@ import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import org.eclipse.jgit.dircache.DirCache;
-import org.eclipse.jgit.dircache.DirCacheCheckout;
-import org.eclipse.jgit.dircache.InvalidPathException;
 import org.eclipse.jgit.errors.AmbiguousObjectException;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.IncorrectObjectTypeException;
@@ -90,6 +88,7 @@ import org.eclipse.jgit.util.FS;
 import org.eclipse.jgit.util.FileUtils;
 import org.eclipse.jgit.util.IO;
 import org.eclipse.jgit.util.RawParseUtils;
+import org.eclipse.jgit.util.SystemReader;
 import org.eclipse.jgit.util.io.SafeBufferedOutputStream;
 
 /**
@@ -384,7 +383,8 @@ public abstract class Repository {
 		try {
 			Object resolved = resolve(rw, revstr);
 			if (resolved instanceof String) {
-				return getRef((String) resolved).getLeaf().getObjectId();
+				final Ref ref = getRef((String)resolved);
+				return ref != null ? ref.getLeaf().getObjectId() : null;
 			} else {
 				return (ObjectId) resolved;
 			}
@@ -1153,11 +1153,11 @@ public abstract class Repository {
 		if (refName.endsWith(".lock")) //$NON-NLS-1$
 			return false;
 
-		// Borrow logic for filterig out invalid paths. These
-		// are also invalid ref
+		// Refs may be stored as loose files so invalid paths
+		// on the local system must also be invalid refs.
 		try {
-			DirCacheCheckout.checkValidPath(refName);
-		} catch (InvalidPathException e) {
+			SystemReader.getInstance().checkPath(refName);
+		} catch (CorruptObjectException e) {
 			return false;
 		}
 
@@ -1273,6 +1273,40 @@ public abstract class Repository {
 		return refName;
 	}
 
+	/**
+	 * @param refName
+	 * @return the remote branch name part of <code>refName</code>, i.e. without
+	 *         the <code>refs/remotes/&lt;remote&gt;</code> prefix, if
+	 *         <code>refName</code> represents a remote tracking branch;
+	 *         otherwise null.
+	 * @since 3.4
+	 */
+	public String shortenRemoteBranchName(String refName) {
+		for (String remote : getRemoteNames()) {
+			String remotePrefix = Constants.R_REMOTES + remote + "/"; //$NON-NLS-1$
+			if (refName.startsWith(remotePrefix))
+				return refName.substring(remotePrefix.length());
+		}
+		return null;
+	}
+
+	/**
+	 * @param refName
+	 * @return the remote name part of <code>refName</code>, i.e. without the
+	 *         <code>refs/remotes/&lt;remote&gt;</code> prefix, if
+	 *         <code>refName</code> represents a remote tracking branch;
+	 *         otherwise null.
+	 * @since 3.4
+	 */
+	public String getRemoteName(String refName) {
+		for (String remote : getRemoteNames()) {
+			String remotePrefix = Constants.R_REMOTES + remote + "/"; //$NON-NLS-1$
+			if (refName.startsWith(remotePrefix))
+				return remote;
+		}
+		return null;
+	}
+
 	/**
 	 * @param refName
 	 * @return a {@link ReflogReader} for the supplied refname, or null if the
@@ -1358,7 +1392,7 @@ public abstract class Repository {
 	 *            $GIT_DIR/MERGE_HEAD or <code>null</code> to delete the file
 	 * @throws IOException
 	 */
-	public void writeMergeHeads(List<ObjectId> heads) throws IOException {
+	public void writeMergeHeads(List<? extends ObjectId> heads) throws IOException {
 		writeHeadsFile(heads, Constants.MERGE_HEAD);
 	}
 
@@ -1555,7 +1589,7 @@ public abstract class Repository {
 	 * @throws FileNotFoundException
 	 * @throws IOException
 	 */
-	private void writeHeadsFile(List<ObjectId> heads, String filename)
+	private void writeHeadsFile(List<? extends ObjectId> heads, String filename)
 			throws FileNotFoundException, IOException {
 		File headsFile = new File(getDirectory(), filename);
 		if (heads != null) {
@@ -1612,4 +1646,13 @@ public abstract class Repository {
 			throws IOException {
 		new RebaseTodoFile(this).writeRebaseTodoFile(path, steps, append);
 	}
+
+	/**
+	 * @return the names of all known remotes
+	 * @since 3.4
+	 */
+	public Set<String> getRemoteNames() {
+		return getConfig()
+				.getSubsections(ConfigConstants.CONFIG_REMOTE_SECTION);
+	}
 }
