@@ -43,6 +43,7 @@
 
 package org.eclipse.jgit.transport;
 
+import static org.eclipse.jgit.transport.GitProtocolConstants.CAPABILITY_ATOMIC;
 import static org.eclipse.jgit.transport.GitProtocolConstants.CAPABILITY_DELETE_REFS;
 import static org.eclipse.jgit.transport.GitProtocolConstants.CAPABILITY_OFS_DELTA;
 import static org.eclipse.jgit.transport.GitProtocolConstants.CAPABILITY_REPORT_STATUS;
@@ -74,6 +75,7 @@ import org.eclipse.jgit.lib.Config;
 import org.eclipse.jgit.lib.Config.SectionParser;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.NullProgressMonitor;
+import org.eclipse.jgit.lib.ObjectChecker;
 import org.eclipse.jgit.lib.ObjectId;
 import org.eclipse.jgit.lib.ObjectIdSubclassMap;
 import org.eclipse.jgit.lib.ObjectInserter;
@@ -160,13 +162,14 @@ public abstract class BaseReceivePack {
 	private boolean expectDataAfterPackFooter;
 
 	/** Should an incoming transfer validate objects? */
-	private boolean checkReceivedObjects;
+	private ObjectChecker objectChecker;
 
 	/** Should an incoming transfer permit create requests? */
 	private boolean allowCreates;
 
 	/** Should an incoming transfer permit delete requests? */
-	private boolean allowDeletes;
+	private boolean allowAnyDeletes;
+	private boolean allowBranchDeletes;
 
 	/** Should an incoming transfer permit non-fast-forward requests? */
 	private boolean allowNonFastForwards;
@@ -221,7 +224,7 @@ public abstract class BaseReceivePack {
 
 	/** Capabilities requested by the client. */
 	private Set<String> enabledCapabilities;
-
+	private Set<ObjectId> clientShallowCommits;
 	private List<ReceiveCommand> commands;
 
 	private StringBuilder advertiseError;
@@ -254,14 +257,16 @@ public abstract class BaseReceivePack {
 		walk = new RevWalk(db);
 
 		final ReceiveConfig cfg = db.getConfig().get(ReceiveConfig.KEY);
-		checkReceivedObjects = cfg.checkReceivedObjects;
+		objectChecker = cfg.newObjectChecker();
 		allowCreates = cfg.allowCreates;
-		allowDeletes = cfg.allowDeletes;
+		allowAnyDeletes = true;
+		allowBranchDeletes = cfg.allowDeletes;
 		allowNonFastForwards = cfg.allowNonFastForwards;
 		allowOfsDelta = cfg.allowOfsDelta;
 		advertiseRefsHook = AdvertiseRefsHook.DEFAULT;
 		refFilter = RefFilter.DEFAULT;
 		advertisedHaves = new HashSet<ObjectId>();
+		clientShallowCommits = new HashSet<ObjectId>();
 	}
 
 	/** Configuration for receive operations. */
@@ -273,18 +278,26 @@ public abstract class BaseReceivePack {
 		};
 
 		final boolean checkReceivedObjects;
+		final boolean allowLeadingZeroFileMode;
+		final boolean safeForWindows;
+		final boolean safeForMacOS;
 
 		final boolean allowCreates;
-
 		final boolean allowDeletes;
-
 		final boolean allowNonFastForwards;
-
 		final boolean allowOfsDelta;
 
 		ReceiveConfig(final Config config) {
-			checkReceivedObjects = config.getBoolean("receive", "fsckobjects", //$NON-NLS-1$ //$NON-NLS-2$
-					false);
+			checkReceivedObjects = config.getBoolean(
+					"receive", "fsckobjects", //$NON-NLS-1$ //$NON-NLS-2$
+					config.getBoolean("transfer", "fsckobjects", false)); //$NON-NLS-1$ //$NON-NLS-2$
+			allowLeadingZeroFileMode = checkReceivedObjects
+					&& config.getBoolean("fsck", "allowLeadingZeroFileMode", false); //$NON-NLS-1$ //$NON-NLS-2$
+			safeForWindows = checkReceivedObjects
+					&& config.getBoolean("fsck", "safeForWindows", false); //$NON-NLS-1$ //$NON-NLS-2$
+			safeForMacOS = checkReceivedObjects
+					&& config.getBoolean("fsck", "safeForMacOS", false); //$NON-NLS-1$ //$NON-NLS-2$
+
 			allowCreates = true;
 			allowDeletes = !config.getBoolean("receive", "denydeletes", false); //$NON-NLS-1$ //$NON-NLS-2$
 			allowNonFastForwards = !config.getBoolean("receive", //$NON-NLS-1$
@@ -292,6 +305,15 @@ public abstract class BaseReceivePack {
 			allowOfsDelta = config.getBoolean("repack", "usedeltabaseoffset", //$NON-NLS-1$ //$NON-NLS-2$
 					true);
 		}
+
+		ObjectChecker newObjectChecker() {
+			if (!checkReceivedObjects)
+				return null;
+			return new ObjectChecker()
+				.setAllowLeadingZeroFileMode(allowLeadingZeroFileMode)
+				.setSafeForWindows(safeForWindows)
+				.setSafeForMacOS(safeForMacOS);
+		}
 	}
 
 	/**
@@ -481,16 +503,29 @@ public abstract class BaseReceivePack {
 	 *         of the connection.
 	 */
 	public boolean isCheckReceivedObjects() {
-		return checkReceivedObjects;
+		return objectChecker != null;
 	}
 
 	/**
 	 * @param check
 	 *            true to enable checking received objects; false to assume all
 	 *            received objects are valid.
+	 * @see #setObjectChecker(ObjectChecker)
 	 */
 	public void setCheckReceivedObjects(final boolean check) {
-		checkReceivedObjects = check;
+		if (check && objectChecker == null)
+			setObjectChecker(new ObjectChecker());
+		else if (!check && objectChecker != null)
+			setObjectChecker(null);
+	}
+
+	/**
+	 * @param impl if non-null the object checking instance to verify each
+	 *        received object with; null to disable object checking.
+	 * @since 3.4
+	 */
+	public void setObjectChecker(ObjectChecker impl) {
+		objectChecker = impl;
 	}
 
 	/** @return true if the client can request refs to be created. */
@@ -508,7 +543,7 @@ public abstract class BaseReceivePack {
 
 	/** @return true if the client can request refs to be deleted. */
 	public boolean isAllowDeletes() {
-		return allowDeletes;
+		return allowAnyDeletes;
 	}
 
 	/**
@@ -516,7 +551,25 @@ public abstract class BaseReceivePack {
 	 *            true to permit delete ref commands to be processed.
 	 */
 	public void setAllowDeletes(final boolean canDelete) {
-		allowDeletes = canDelete;
+		allowAnyDeletes = canDelete;
+	}
+
+	/**
+	 * @return true if the client can delete from {@code refs/heads/}.
+	 * @since 3.6
+	 */
+	public boolean isAllowBranchDeletes() {
+		return allowBranchDeletes;
+	}
+
+	/**
+	 * @param canDelete
+	 *            true to permit deletion of branches from the
+	 *            {@code refs/heads/} namespace.
+	 * @since 3.6
+	 */
+	public void setAllowBranchDeletes(boolean canDelete) {
+		allowBranchDeletes = canDelete;
 	}
 
 	/**
@@ -739,6 +792,18 @@ public abstract class BaseReceivePack {
 		throw new IllegalStateException(JGitText.get().packSizeNotSetYet);
 	}
 
+	/**
+	 * Get the commits from the client's shallow file.
+	 *
+	 * @return if the client is a shallow repository, the list of edge commits
+	 *     that define the client's shallow boundary. Empty set if the client
+	 *     is earlier than Git 1.9, or is a full clone.
+	 * @since 3.5
+	 */
+	protected Set<ObjectId> getClientShallowCommits() {
+		return clientShallowCommits;
+	}
+
 	/** @return true if any commands to be executed have been read. */
 	protected boolean hasCommands() {
 		return !commands.isEmpty();
@@ -864,6 +929,8 @@ public abstract class BaseReceivePack {
 		adv.advertiseCapability(CAPABILITY_SIDE_BAND_64K);
 		adv.advertiseCapability(CAPABILITY_DELETE_REFS);
 		adv.advertiseCapability(CAPABILITY_REPORT_STATUS);
+		if (db.getRefDatabase().performsAtomicTransactions())
+			adv.advertiseCapability(CAPABILITY_ATOMIC);
 		if (allowOfsDelta)
 			adv.advertiseCapability(CAPABILITY_OFS_DELTA);
 		adv.send(getAdvertisedOrDefaultRefs());
@@ -892,6 +959,11 @@ public abstract class BaseReceivePack {
 			if (line == PacketLineIn.END)
 				break;
 
+			if (line.length() >= 48 && line.startsWith("shallow ")) { //$NON-NLS-1$
+				clientShallowCommits.add(ObjectId.fromString(line.substring(8, 48)));
+				continue;
+			}
+
 			if (commands.isEmpty()) {
 				final FirstLine firstLine = new FirstLine(line);
 				enabledCapabilities = firstLine.getCapabilities();
@@ -983,7 +1055,7 @@ public abstract class BaseReceivePack {
 			parser.setCheckEofAfterPackFooter(!biDirectionalPipe
 					&& !isExpectDataAfterPackFooter());
 			parser.setExpectDataAfterPackFooter(isExpectDataAfterPackFooter());
-			parser.setObjectChecking(isCheckReceivedObjects());
+			parser.setObjectChecker(objectChecker);
 			parser.setLockMessage(lockMsg);
 			parser.setMaxObjectSizeLimit(maxObjectSizeLimit);
 			packLock = parser.parse(receiving, resolving);
@@ -999,12 +1071,19 @@ public abstract class BaseReceivePack {
 
 	private boolean needCheckConnectivity() {
 		return isCheckReceivedObjects()
-				|| isCheckReferencedObjectsAreReachable();
+				|| isCheckReferencedObjectsAreReachable()
+				|| !getClientShallowCommits().isEmpty();
 	}
 
 	private void checkConnectivity() throws IOException {
 		ObjectIdSubclassMap<ObjectId> baseObjects = null;
 		ObjectIdSubclassMap<ObjectId> providedObjects = null;
+		ProgressMonitor checking = NullProgressMonitor.INSTANCE;
+		if (sideBand) {
+			SideBandProgressMonitor m = new SideBandProgressMonitor(msgOut);
+			m.setDelayStart(750, TimeUnit.MILLISECONDS);
+			checking = m;
+		}
 
 		if (checkReferencedIsReachable) {
 			baseObjects = parser.getBaseObjectIds();
@@ -1040,8 +1119,10 @@ public abstract class BaseReceivePack {
 			}
 		}
 
+		checking.beginTask(JGitText.get().countingObjects, ProgressMonitor.UNKNOWN);
 		RevCommit c;
 		while ((c = ow.next()) != null) {
+			checking.update(1);
 			if (providedObjects != null //
 					&& !c.has(RevFlag.UNINTERESTING) //
 					&& !providedObjects.contains(c))
@@ -1050,6 +1131,7 @@ public abstract class BaseReceivePack {
 
 		RevObject o;
 		while ((o = ow.nextObject()) != null) {
+			checking.update(1);
 			if (o.has(RevFlag.UNINTERESTING))
 				continue;
 
@@ -1063,6 +1145,7 @@ public abstract class BaseReceivePack {
 			if (o instanceof RevBlob && !db.hasObject(o))
 				throw new MissingObjectException(o, Constants.TYPE_BLOB);
 		}
+		checking.endTask();
 
 		if (baseObjects != null) {
 			for (ObjectId id : baseObjects) {
@@ -1080,12 +1163,18 @@ public abstract class BaseReceivePack {
 			if (cmd.getResult() != Result.NOT_ATTEMPTED)
 				continue;
 
-			if (cmd.getType() == ReceiveCommand.Type.DELETE
-					&& !isAllowDeletes()) {
-				// Deletes are not supported on this repository.
-				//
-				cmd.setResult(Result.REJECTED_NODELETE);
-				continue;
+			if (cmd.getType() == ReceiveCommand.Type.DELETE) {
+				if (!isAllowDeletes()) {
+					// Deletes are not supported on this repository.
+					cmd.setResult(Result.REJECTED_NODELETE);
+					continue;
+				}
+				if (!isAllowBranchDeletes()
+						&& ref.getName().startsWith(Constants.R_HEADS)) {
+					// Branches cannot be deleted, but other refs can.
+					cmd.setResult(Result.REJECTED_NODELETE);
+					continue;
+				}
 			}
 
 			if (cmd.getType() == ReceiveCommand.Type.CREATE) {
@@ -1191,6 +1280,29 @@ public abstract class BaseReceivePack {
 		}
 	}
 
+	/**
+	 * @return if any commands have been rejected so far.
+	 * @since 3.6
+	 */
+	protected boolean anyRejects() {
+		for (ReceiveCommand cmd : commands) {
+			if (cmd.getResult() != Result.NOT_ATTEMPTED && cmd.getResult() != Result.OK)
+				return true;
+		}
+		return false;
+	}
+
+	/**
+	 * Set the result to fail for any command that was not processed yet.
+	 * @since 3.6
+	 */
+	protected void failPendingCommands() {
+		for (ReceiveCommand cmd : commands) {
+			if (cmd.getResult() == Result.NOT_ATTEMPTED)
+				cmd.setResult(Result.REJECTED_OTHER_REASON, JGitText.get().transactionAborted);
+		}
+	}
+
 	/**
 	 * Filter the list of commands according to result.
 	 *
