@@ -52,6 +52,7 @@ import static org.eclipse.jgit.util.HttpSupport.HDR_CONTENT_ENCODING;
 import static org.eclipse.jgit.util.HttpSupport.HDR_CONTENT_TYPE;
 import static org.eclipse.jgit.util.HttpSupport.HDR_PRAGMA;
 import static org.eclipse.jgit.util.HttpSupport.HDR_USER_AGENT;
+import static org.eclipse.jgit.util.HttpSupport.HDR_WWW_AUTHENTICATE;
 import static org.eclipse.jgit.util.HttpSupport.METHOD_GET;
 import static org.eclipse.jgit.util.HttpSupport.METHOD_POST;
 
@@ -245,7 +246,9 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 
 	private boolean useSmartHttp = true;
 
-	private HttpAuthMethod authMethod = HttpAuthMethod.NONE;
+	private HttpAuthMethod authMethod = HttpAuthMethod.Type.NONE.method(null);
+
+	private Map<String, String> headers;
 
 	TransportHttp(final Repository local, final URIish uri)
 			throws NotSupportedException {
@@ -425,6 +428,18 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 		// No explicit connections are maintained.
 	}
 
+	/**
+	 * Set additional headers on the HTTP connection
+	 *
+	 * @param headers
+	 *            a map of name:values that are to be set as headers on the HTTP
+	 *            connection
+	 * @since 3.4
+	 */
+	public void setAdditionalHeaders(Map<String, String> headers) {
+		this.headers = headers;
+	}
+
 	private HttpConnection connect(final String service)
 			throws TransportException, NotSupportedException {
 		final URL u;
@@ -460,6 +475,13 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 				final int status = HttpSupport.response(conn);
 				switch (status) {
 				case HttpConnection.HTTP_OK:
+					// Check if HttpConnection did some authentication in the
+					// background (e.g Kerberos/SPNEGO).
+					// That may not work for streaming requests and jgit
+					// explicit authentication would be required
+					if (authMethod.getType() == HttpAuthMethod.Type.NONE
+							&& conn.getHeaderField(HDR_WWW_AUTHENTICATE) != null)
+						authMethod = HttpAuthMethod.scanResponse(conn);
 					return conn;
 
 				case HttpConnection.HTTP_NOT_FOUND:
@@ -468,17 +490,20 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 
 				case HttpConnection.HTTP_UNAUTHORIZED:
 					authMethod = HttpAuthMethod.scanResponse(conn);
-					if (authMethod == HttpAuthMethod.NONE)
+					if (authMethod.getType() == HttpAuthMethod.Type.NONE)
 						throw new TransportException(uri, MessageFormat.format(
 								JGitText.get().authenticationNotSupported, uri));
 					CredentialsProvider credentialsProvider = getCredentialsProvider();
+					if (credentialsProvider == null)
+						throw new TransportException(uri,
+								JGitText.get().noCredentialsProvider);
+					if (authAttempts > 1)
+						credentialsProvider.reset(uri);
 					if (3 < authAttempts
 							|| !authMethod.authorize(uri, credentialsProvider)) {
-						credentialsProvider.reset(uri);
 						throw new TransportException(uri,
 								JGitText.get().notAuthorized);
 					}
-					credentialsProvider.reset(uri);
 					authAttempts++;
 					continue;
 
@@ -533,6 +558,10 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 			conn.setConnectTimeout(effTimeOut);
 			conn.setReadTimeout(effTimeOut);
 		}
+		if (this.headers != null && !this.headers.isEmpty()) {
+			for (Map.Entry<String, String> entry : this.headers.entrySet())
+				conn.setRequestProperty(entry.getKey(), entry.getValue());
+		}
 		authMethod.configureRequest(conn);
 		return conn;
 	}
@@ -598,15 +627,15 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 	}
 
 	class HttpObjectDB extends WalkRemoteObjectDatabase {
-		private final URL objectsUrl;
+		private final URL httpObjectsUrl;
 
 		HttpObjectDB(final URL b) {
-			objectsUrl = b;
+			httpObjectsUrl = b;
 		}
 
 		@Override
 		URIish getURI() {
-			return new URIish(objectsUrl);
+			return new URIish(httpObjectsUrl);
 		}
 
 		@Override
@@ -629,7 +658,7 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 		@Override
 		WalkRemoteObjectDatabase openAlternate(final String location)
 				throws IOException {
-			return new HttpObjectDB(new URL(objectsUrl, location));
+			return new HttpObjectDB(new URL(httpObjectsUrl, location));
 		}
 
 		@Override
@@ -657,7 +686,7 @@ public class TransportHttp extends HttpTransport implements WalkTransport,
 
 		@Override
 		FileStream open(final String path) throws IOException {
-			final URL base = objectsUrl;
+			final URL base = httpObjectsUrl;
 			final URL u = new URL(base, path);
 			final HttpConnection c = httpOpen(u);
 			switch (HttpSupport.response(c)) {
