@@ -328,13 +328,13 @@ public abstract class AbstractTreeIterator {
 	 *            position to start reading the raw buffer.
 	 * @param end
 	 *            one past the end of the raw buffer (length is end - pos).
-	 * @param mode
+	 * @param pathMode
 	 *            the mode of the path.
 	 * @return -1 if this entry sorts first; 0 if the entries are equal; 1 if
 	 *         p's entry sorts first.
 	 */
-	public int pathCompare(byte[] buf, int pos, int end, int mode) {
-		return pathCompare(buf, pos, end, mode, 0);
+	public int pathCompare(byte[] buf, int pos, int end, int pathMode) {
+		return pathCompare(buf, pos, end, pathMode, 0);
 	}
 
 	private int pathCompare(byte[] b, int bPos, int bEnd, int bMode, int aPos) {
