@@ -63,6 +63,8 @@ import java.util.Collections;
 import java.util.Comparator;
 
 import org.eclipse.jgit.api.errors.JGitInternalException;
+import org.eclipse.jgit.attributes.AttributesNode;
+import org.eclipse.jgit.attributes.AttributesRule;
 import org.eclipse.jgit.diff.RawText;
 import org.eclipse.jgit.dircache.DirCache;
 import org.eclipse.jgit.dircache.DirCacheEntry;
@@ -70,8 +72,8 @@ import org.eclipse.jgit.dircache.DirCacheIterator;
 import org.eclipse.jgit.errors.CorruptObjectException;
 import org.eclipse.jgit.errors.MissingObjectException;
 import org.eclipse.jgit.errors.NoWorkTreeException;
+import org.eclipse.jgit.ignore.FastIgnoreRule;
 import org.eclipse.jgit.ignore.IgnoreNode;
-import org.eclipse.jgit.ignore.IgnoreRule;
 import org.eclipse.jgit.internal.JGitText;
 import org.eclipse.jgit.lib.Constants;
 import org.eclipse.jgit.lib.CoreConfig;
@@ -133,6 +135,9 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	/** If there is a .gitignore file present, the parsed rules from it. */
 	private IgnoreNode ignoreNode;
 
+	/** If there is a .gitattributes file present, the parsed rules from it. */
+	private AttributesNode attributesNode;
+
 	/** Repository that is the root level being iterated over */
 	protected Repository repository;
 
@@ -142,6 +147,19 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	/** The offset of the content id in {@link #idBuffer()} */
 	private int contentIdOffset;
 
+	/**
+	 * Holds the {@link AttributesNode} that is stored in
+	 * $GIT_DIR/info/attributes file.
+	 */
+	private AttributesNode infoAttributeNode;
+
+	/**
+	 * Holds the {@link AttributesNode} that is stored in global attribute file.
+	 *
+	 * @see CoreConfig#getAttributesFile()
+	 */
+	private AttributesNode globalAttributeNode;
+
 	/**
 	 * Create a new iterator with no parent.
 	 *
@@ -185,6 +203,8 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	protected WorkingTreeIterator(final WorkingTreeIterator p) {
 		super(p);
 		state = p.state;
+		infoAttributeNode = p.infoAttributeNode;
+		globalAttributeNode = p.globalAttributeNode;
 	}
 
 	/**
@@ -204,6 +224,10 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		else
 			entry = null;
 		ignoreNode = new RootIgnoreNode(entry, repo);
+
+		infoAttributeNode = new InfoAttributesNode(repo);
+
+		globalAttributeNode = new GlobalAttributesNode(repo);
 	}
 
 	/**
@@ -401,11 +425,11 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		}
 	}
 
-	private boolean isBinary(byte[] content, int sz) {
+	private static boolean isBinary(byte[] content, int sz) {
 		return RawText.isBinary(content, sz);
 	}
 
-	private boolean isBinary(Entry entry) throws IOException {
+	private static boolean isBinary(Entry entry) throws IOException {
 		InputStream in = entry.openInputStream();
 		try {
 			return RawText.isBinary(in);
@@ -414,7 +438,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		}
 	}
 
-	private ByteBuffer filterClean(byte[] src, int n)
+	private static ByteBuffer filterClean(byte[] src, int n)
 			throws IOException {
 		InputStream in = new ByteArrayInputStream(src);
 		try {
@@ -424,7 +448,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		}
 	}
 
-	private InputStream filterClean(InputStream in) {
+	private static InputStream filterClean(InputStream in) {
 		return new EolCanonicalizingInputStream(in, true);
 	}
 
@@ -573,6 +597,23 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	 *             a relevant ignore rule file exists but cannot be read.
 	 */
 	protected boolean isEntryIgnored(final int pLen) throws IOException {
+		return isEntryIgnored(pLen, false);
+	}
+
+	/**
+	 * Determine if the entry path is ignored by an ignore rule. Consider
+	 * possible rule negation from child iterator.
+	 *
+	 * @param pLen
+	 *            the length of the path in the path buffer.
+	 * @param negatePrevious
+	 *            true if the previous matching iterator rule was negation
+	 * @return true if the entry is ignored by an ignore rule.
+	 * @throws IOException
+	 *             a relevant ignore rule file exists but cannot be read.
+	 */
+	private boolean isEntryIgnored(final int pLen, boolean negatePrevious)
+			throws IOException {
 		IgnoreNode rules = getIgnoreNode();
 		if (rules != null) {
 			// The ignore code wants path to start with a '/' if possible.
@@ -583,17 +624,23 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 			if (0 < pOff)
 				pOff--;
 			String p = TreeWalk.pathOf(path, pOff, pLen);
-			switch (rules.isIgnored(p, FileMode.TREE.equals(mode))) {
+			switch (rules.isIgnored(p, FileMode.TREE.equals(mode),
+					negatePrevious)) {
 			case IGNORED:
 				return true;
 			case NOT_IGNORED:
 				return false;
 			case CHECK_PARENT:
+				negatePrevious = false;
+				break;
+			case CHECK_PARENT_NEGATE_FIRST_MATCH:
+				negatePrevious = true;
 				break;
 			}
 		}
 		if (parent instanceof WorkingTreeIterator)
-			return ((WorkingTreeIterator) parent).isEntryIgnored(pLen);
+			return ((WorkingTreeIterator) parent).isEntryIgnored(pLen,
+					negatePrevious);
 		return false;
 	}
 
@@ -603,6 +650,56 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		return ignoreNode;
 	}
 
+	/**
+	 * Retrieves the {@link AttributesNode} for the current entry.
+	 *
+	 * @return {@link AttributesNode} for the current entry.
+	 * @throws IOException
+	 *             if an error is raised while parsing the .gitattributes file
+	 * @since 3.7
+	 */
+	public AttributesNode getEntryAttributesNode() throws IOException {
+		if (attributesNode instanceof PerDirectoryAttributesNode)
+			attributesNode = ((PerDirectoryAttributesNode) attributesNode)
+					.load();
+		return attributesNode;
+	}
+
+	/**
+	 * Retrieves the {@link AttributesNode} that holds the information located
+	 * in $GIT_DIR/info/attributes file.
+	 *
+	 * @return the {@link AttributesNode} that holds the information located in
+	 *         $GIT_DIR/info/attributes file.
+	 * @throws IOException
+	 *             if an error is raised while parsing the attributes file
+	 * @since 3.7
+	 */
+	public AttributesNode getInfoAttributesNode() throws IOException {
+		if (infoAttributeNode instanceof InfoAttributesNode)
+			infoAttributeNode = ((InfoAttributesNode) infoAttributeNode).load();
+		return infoAttributeNode;
+	}
+
+	/**
+	 * Retrieves the {@link AttributesNode} that holds the information located
+	 * in system-wide file.
+	 *
+	 * @return the {@link AttributesNode} that holds the information located in
+	 *         system-wide file.
+	 * @throws IOException
+	 *             IOException if an error is raised while parsing the
+	 *             attributes file
+	 * @see CoreConfig#getAttributesFile()
+	 * @since 3.7
+	 */
+	public AttributesNode getGlobalAttributesNode() throws IOException {
+		if (globalAttributeNode instanceof GlobalAttributesNode)
+			globalAttributeNode = ((GlobalAttributesNode) globalAttributeNode)
+					.load();
+		return globalAttributeNode;
+	}
+
 	private static final Comparator<Entry> ENTRY_CMP = new Comparator<Entry>() {
 		public int compare(final Entry o1, final Entry o2) {
 			final byte[] a = o1.encodedName;
@@ -656,6 +753,8 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 				continue;
 			if (Constants.DOT_GIT_IGNORE.equals(name))
 				ignoreNode = new PerDirectoryIgnoreNode(e);
+			if (Constants.DOT_GIT_ATTRIBUTES.equals(name))
+				attributesNode = new PerDirectoryAttributesNode(e);
 			if (i != o)
 				entries[o] = e;
 			e.encodeName(nameEncoder);
@@ -668,6 +767,8 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		ptr = 0;
 		if (!eof())
 			parseEntry();
+		else if (pathLen == 0) // see bug 445363
+			pathLen = pathOffset;
 	}
 
 	/**
@@ -801,6 +902,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	 * @return true if content is most likely different.
 	 * @deprecated Use {@link #isModified(DirCacheEntry, boolean, ObjectReader)}
 	 */
+	@Deprecated
 	public boolean isModified(DirCacheEntry entry, boolean forceContentCheck) {
 		try {
 			return isModified(entry, forceContentCheck,
@@ -831,6 +933,8 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 	 */
 	public boolean isModified(DirCacheEntry entry, boolean forceContentCheck,
 			ObjectReader reader) throws IOException {
+		if (entry == null)
+			return !FileMode.MISSING.equals(getEntryFileMode());
 		MetadataDiff diff = compareMetadata(entry);
 		switch (diff) {
 		case DIFFER_BY_TIMESTAMP:
@@ -979,7 +1083,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		return FS.detect().normalize(RawParseUtils.decode(content));
 	}
 
-	private long computeLength(InputStream in) throws IOException {
+	private static long computeLength(InputStream in) throws IOException {
 		// Since we only care about the length, use skip. The stream
 		// may be able to more efficiently wade through its data.
 		//
@@ -1127,7 +1231,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		final Entry entry;
 
 		PerDirectoryIgnoreNode(Entry entry) {
-			super(Collections.<IgnoreRule> emptyList());
+			super(Collections.<FastIgnoreRule> emptyList());
 			this.entry = entry;
 		}
 
@@ -1182,7 +1286,7 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 			return r.getRules().isEmpty() ? null : r;
 		}
 
-		private void loadRulesFromFile(IgnoreNode r, File exclude)
+		private static void loadRulesFromFile(IgnoreNode r, File exclude)
 				throws FileNotFoundException, IOException {
 			if (FS.DETECTED.exists(exclude)) {
 				FileInputStream in = new FileInputStream(exclude);
@@ -1195,6 +1299,90 @@ public abstract class WorkingTreeIterator extends AbstractTreeIterator {
 		}
 	}
 
+	/** Magic type indicating we know rules exist, but they aren't loaded. */
+	private static class PerDirectoryAttributesNode extends AttributesNode {
+		final Entry entry;
+
+		PerDirectoryAttributesNode(Entry entry) {
+			super(Collections.<AttributesRule> emptyList());
+			this.entry = entry;
+		}
+
+		AttributesNode load() throws IOException {
+			AttributesNode r = new AttributesNode();
+			InputStream in = entry.openInputStream();
+			try {
+				r.parse(in);
+			} finally {
+				in.close();
+			}
+			return r.getRules().isEmpty() ? null : r;
+		}
+	}
+
+	/**
+	 * Attributes node loaded from global system-wide file.
+	 */
+	private static class GlobalAttributesNode extends AttributesNode {
+		final Repository repository;
+
+		GlobalAttributesNode(Repository repository) {
+			this.repository = repository;
+		}
+
+		AttributesNode load() throws IOException {
+			AttributesNode r = new AttributesNode();
+
+			FS fs = repository.getFS();
+			String path = repository.getConfig().get(CoreConfig.KEY)
+					.getAttributesFile();
+			if (path != null) {
+				File attributesFile;
+				if (path.startsWith("~/")) //$NON-NLS-1$
+					attributesFile = fs.resolve(fs.userHome(),
+							path.substring(2));
+				else
+					attributesFile = fs.resolve(null, path);
+				loadRulesFromFile(r, attributesFile);
+			}
+			return r.getRules().isEmpty() ? null : r;
+		}
+	}
+
+	/** Magic type indicating there may be rules for the top level. */
+	private static class InfoAttributesNode extends AttributesNode {
+		final Repository repository;
+
+		InfoAttributesNode(Repository repository) {
+			this.repository = repository;
+		}
+
+		AttributesNode load() throws IOException {
+			AttributesNode r = new AttributesNode();
+
+			FS fs = repository.getFS();
+
+			File attributes = fs.resolve(repository.getDirectory(),
+					"info/attributes"); //$NON-NLS-1$
+			loadRulesFromFile(r, attributes);
+
+			return r.getRules().isEmpty() ? null : r;
+		}
+
+	}
+
+	private static void loadRulesFromFile(AttributesNode r, File attrs)
+			throws FileNotFoundException, IOException {
+		if (attrs.exists()) {
+			FileInputStream in = new FileInputStream(attrs);
+			try {
+				r.parse(in);
+			} finally {
+				in.close();
+			}
+		}
+	}
+
 	private static final class IteratorState {
 		/** Options used to process the working tree. */
 		final WorkingTreeOptions options;
